//
//  ES1Renderer.m
//  FlatShaded
//
//  Created by Edward Patel on 2009-10-04.
//  Copyright Memention AB 2009. All rights reserved.
//

#import "ES1Renderer.h"

@implementation ES1Renderer

// Create an ES 1.1 context
- (id) init
{
	if (self = [super init])
	{
		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
        
        if (!context || ![EAGLContext setCurrentContext:context])
		{
            [self release];
            return nil;
        }
		
		// Create default framebuffer object. The backing will be allocated for the current layer in -resizeFromLayer
		glGenFramebuffersOES(1, &defaultFramebuffer);
		glGenRenderbuffersOES(1, &colorRenderbuffer);
				
		glBindFramebufferOES(GL_FRAMEBUFFER_OES, defaultFramebuffer);
		glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
		glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, 
									 GL_COLOR_ATTACHMENT0_OES, 
									 GL_RENDERBUFFER_OES, colorRenderbuffer);
	}
	
	return self;
}

- (void) render:(int)useNormalList
{
    // Replace the implementation of this method to do your own custom drawing
    
    static const GLfloat squareVertices[] = {
		-1, -1.2, -1,
		+1, +1, -1,
		+1, -1, -1,

		+1, +1, +1,
		+1, -1, +1,
		
		-1, +1, +1,
		-1, -1, +1,

		-1, +1, -1,
		-1, -1, -1,

		+1, +1, -1,
	};

	static const GLfloat normals_org[] = {
		+0, +0, -1,
		+0, +0, -1,
		+0, +0, -1,
		
		+1, +0, +0,
		+1, +0, +0,
		
		+0, +0, +1,
		+0, +0, +1,
		
		-1, +0, +0,
		-1, +0, +0,
		
		+0, +0, -1,
	};

	static GLfloat normals_1[] = {
		+1, +0, +0,
		
		+1, +0, +0,
		+1, +0, +0,
		
		+1, +0, +0,
		+1, +0, +0,
		
		+1, +0, +0,
		+1, +0, +0,
		
		+1, +0, +0,
		+1, +0, +0,

		+0, +0, -1, // <<<<<<<<<<<<<<
		
		+1, +0, +0,
		
		+1, +0, +0,
		+1, +0, +0,
		
		+1, +0, +0,
		+1, +0, +0,
		
		+1, +0, +0,
		+1, +0, +0,
		
		+1, +0, +0,
		+1, +0, +0,
	};

	static float transY = 0.0f;
	
	// This application only creates a single context which is already set current at this point.
	// This call is redundant, but needed if dealing with multiple contexts.
    [EAGLContext setCurrentContext:context];
    
	// This application only creates a single default framebuffer which is already bound at this point.
	// This call is redundant, but needed if dealing with multiple framebuffers.
    glBindFramebufferOES(GL_FRAMEBUFFER_OES, defaultFramebuffer);
    glViewport(0, 0, backingWidth, backingHeight);
    
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glShadeModel(GL_FLAT);
	
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	glFrustumf(-1, 1, -1.5, 1.5, 1, 100.0);
    glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	glTranslatef(0, 0, -3);
	
    glRotatef(30, 1, 0, 0);
    glRotatef(transY, 0, 1, 0);
	transY += 0.5f;
	
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
	glColor4f(1, 0, 0, 1);
    glVertexPointer(3, GL_FLOAT, 0, squareVertices);
	switch (useNormalList) {
		case 0:
			glNormalPointer(GL_FLOAT, 0, normals_org);
			break;
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
			glNormalPointer(GL_FLOAT, 0, &normals_1[9*3-useNormalList*3]);
			break;
		default:
			break;
	}
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
	//glColorPointer(4, GL_UNSIGNED_BYTE, 0, squareColors);
    //glEnableClientState(GL_COLOR_ARRAY);
    
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 10);
    
	// This application only creates a single color renderbuffer which is already bound at this point.
	// This call is redundant, but needed if dealing with multiple renderbuffers.
    glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
    [context presentRenderbuffer:GL_RENDERBUFFER_OES];
}

- (BOOL) resizeFromLayer:(CAEAGLLayer *)layer
{	
	// Allocate color buffer backing based on the current layer size
    glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
    [context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:layer];
	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &backingWidth);
    glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &backingHeight);
	
	if (!depthRenderbuffer) {
		glGenRenderbuffersOES(1, &depthRenderbuffer);
		glBindRenderbufferOES(GL_RENDERBUFFER_OES, depthRenderbuffer);
		glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT16_OES, backingWidth, backingHeight);
		glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, depthRenderbuffer);
	}
	
    if (glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES)
	{
		NSLog(@"Failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
        return NO;
    }
    
    return YES;
}

- (void) dealloc
{
	// Tear down GL
	if (defaultFramebuffer)
	{
		glDeleteFramebuffersOES(1, &defaultFramebuffer);
		defaultFramebuffer = 0;
	}

	if (colorRenderbuffer)
	{
		glDeleteRenderbuffersOES(1, &colorRenderbuffer);
		colorRenderbuffer = 0;
	}
	
	// Tear down context
	if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
	
	[context release];
	context = nil;
	
	[super dealloc];
}

@end
